#include "helper.h"

#include "math.h"

#include "OsiSolverInterface.hpp"
#include "CoinPackedMatrix.hpp"

double precision = 1e-3;

bool checkEquality(double v1, double v2)
{
	if (!v2 && fabs(v1) < precision)
		return true;

	return (fabs(v1 / v2 - 1) < precision);
}

void check(bool b)
{
	if (!b) {
		printf("fail\n");
		//assert(false);
	}
}

void logString(FILE *log, const char *string)
{
	fprintf(log, "%s\n", string);
	printf("%s\n", string);
}

void logProblem(FILE *log, OsiSolverInterface *s)
{
	int i;
	double inf = s->getInfinity();

	// Report objective function.
	if (s->getObjSense() == 1) {
		fprintf(log, "Minimize\n");
	}
	else if (s->getObjSense() == -1) {
		fprintf(log, "Maximize\n");
	}
	const double *obj = s->getObjCoefficients();
	fprintf(log, "obj: ");
	for (i = 0; i < s->getNumCols(); ++i) {
		if (obj[i])
			fprintf(log, "%+.2f x%d ", obj[i], i); 
	}
	fprintf(log, "\n");

	// Report constraints.
	const double *row_lower = s->getRowLower();
	const double *row_upper = s->getRowUpper();
	const CoinPackedMatrix *matrix = s->getMatrixByRow();
	for (int j = 0; j < matrix->getNumRows(); ++j) {
		const CoinShallowPackedVector vector = matrix->getVector(j);
		fprintf(log, "c%d: ", j);
		if (row_lower[j] > -inf)
			fprintf(log, "%.2f <= ", row_lower[j]);
		for (int i = 0; i < vector.getNumElements(); ++i) {
			fprintf(log, "%+.2f x%d ", vector.getElements()[i], vector.getIndices()[i]);
		}
		if (row_upper[j] < inf)
			fprintf(log, "<= %.2f ", row_upper[j]);
		fprintf(log, "\n");
	}

	const double *col_lower = s->getColLower();
	const double *col_upper = s->getColUpper();
	for (i = 0; i < s->getNumCols(); ++i) {
		if (col_lower[i] <= -inf && col_upper[i] >= inf)
			continue;
		if (col_lower[i] > -inf)
			fprintf(log, "%.2f <= ", col_lower[i]);
		fprintf(log, "x%d ", i);
		if (col_upper[i] < inf)
			fprintf(log, "%.2f <= ", col_upper[i]);
		fprintf(log, "\n");
	}
}

void logSolution(FILE *log, OsiSolverInterface *s)
{
	char buffer[256];
	bool optimal = s->isProvenOptimal();
	if (!optimal) {
		sprintf(buffer, "no optimal solution.");
		logString(log, buffer);
	}

	bool primal_infeasible = s->isProvenPrimalInfeasible();
	if (primal_infeasible) {
		sprintf(buffer, "no primal feasible solution.");
		logString(log, buffer);
	}

	bool dual_infeasible = s->isProvenDualInfeasible();
	if (dual_infeasible) {
		sprintf(buffer, "no dual feasible solution.");
		logString(log, buffer);
	}

	if (optimal) {
		const double *solution = s->getColSolution();
		fprintf(log, "solution: ");
		for (int i = 0; i < s->getNumCols(); ++i) {
			fprintf(log, "%.2f ", solution[i]);
		}
		fprintf(log, "\n");
		fprintf(log, "activity:\n");
		const double *activity = s->getRowActivity();
		for (int j = 0; j < s->getNumRows(); ++j) {
			fprintf(log, "c%d: %.2f\n", j, activity[j]);
		}
	}
}
