#include "test10.h"

#include "helper.h"
#include "CoinPackedVector.hpp"
#include "OsiSolverInterface.hpp"

bool test10_solve(OsiSolverInterface *s, const char *solvername)
{
	bool ret = true;
	std::string test_name("test10");

	std::string log_path = std::string("test/") + solvername + "/" + test_name + ".log";
	FILE *log = fopen(log_path.c_str(), "w");

	logString(log, test_name.c_str());

	double inf = s->getInfinity();

	int numcols = 2;
	int numrows = 2;
	const int start[] = {0, 2, 4};
	const int index[] = {0, 1, 0, 1};
	const double value[] = {2, 1, 1, 3};
	const double collb[] = {0, 0};
	const double colub[] = {inf, inf};
	double obj[] = {3, 1};
	char rowsen[] = {'R', 'R'};
	double rowrhs[] = {10, 15};
	double rowrng[] = {10, 15};

	s->loadProblem(numcols, numrows, start, index, value, collb, colub, obj, rowsen, rowrhs, rowrng);

	s->setObjSense(-1);

	s->writeMps("test");

	s->resolve();

	ret = ret && s->isProvenOptimal();
	ret = ret && !s->isProvenPrimalInfeasible();
	ret = ret && !s->isProvenDualInfeasible();
	ret = ret && checkEquality(s->getColSolution()[0], 5);
	ret = ret && checkEquality(s->getColSolution()[1], 0);

	logProblem(log, s);
	logSolution(log, s);

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();

	ret = ret && s->isProvenOptimal();
	ret = ret && !s->isProvenPrimalInfeasible();
	ret = ret && !s->isProvenDualInfeasible();
	ret = ret && checkEquality(s->getColSolution()[0], 3);
	ret = ret && checkEquality(s->getColSolution()[1], 4);

	logProblem(log, s);
	logSolution(log, s);

	fclose(log);
	return ret;
}
