#include "test8.h"

#include "helper.h"
#include "CoinPackedVector.hpp"
#include "OsiSolverInterface.hpp"

bool test8_solve(OsiSolverInterface *s, const char *solvername)
{
	bool ret = true;
	std::string test_name("test8");

	std::string log_path = std::string("test/") + solvername + "/" + test_name + ".log";
	FILE *log = fopen(log_path.c_str(), "w");

	logString(log, test_name.c_str());

	double inf = s->getInfinity();

	CoinPackedVector empty;

	s->addCol(empty, -inf, inf, 3);
	s->addCol(empty, -inf, inf, 1);

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	s->addRow(r0, 0, 10);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	s->addRow(r1, 0, 15);

	s->setObjSense(-1);

	s->writeMps("test");

	s->resolve();

	ret = ret && s->isProvenOptimal();
	ret = ret && !s->isProvenPrimalInfeasible();
	ret = ret && !s->isProvenDualInfeasible();
	ret = ret && checkEquality(s->getColSolution()[0], 6);
	ret = ret && checkEquality(s->getColSolution()[1], -2);

	logProblem(log, s);
	logSolution(log, s);

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();

	ret = ret && s->isProvenOptimal();
	ret = ret && !s->isProvenPrimalInfeasible();
	ret = ret && !s->isProvenDualInfeasible();
	ret = ret && checkEquality(s->getColSolution()[0], 3);
	ret = ret && checkEquality(s->getColSolution()[1], 4);

	logProblem(log, s);
	logSolution(log, s);

	fclose(log);
	return ret;
}
